/* lpfilter
 *
 * $Id$
 *
 * $Log$
 */
#include <errno.h>
#include <stdio.h>
#include <strings.h>
#include <syslog.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#define LINELENGTH 2000					    /* max length of program line */
  char inbuf [LINELENGTH],				    /* input buffer */
       outbuf [LINELENGTH * 2];				    /* and output buffer */
#define MAXLINES 54					    /* number of lines on page */

unsigned char translate [256] =				    /* translate table for IBM to LaserJet */
{
  0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
  0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,
  0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
  0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
  0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f,
  0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f,
  0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
  0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f,
  0xb4, 0xcf, 0xc5, 0xc0, 0xcc, 0xc8, 0xd4, 0xb5, 0xc1, 0xcd, 0xc9, 0xdd, 0xd1, 0xd9, 0xd8, 0xd0,
  0xdc, 0xd7, 0xd3, 0xc2, 0xce, 0xca, 0xc3, 0xcb, 0xef, 0xda, 0xdb, 0xbf, 0xbb, 0xbc, 0xfc, 0xbe,
  0xc4, 0xd5, 0xc6, 0xc7, 0xb7, 0xb6, 0xf9, 0xfa, 0xb9, 0xfc, 0xfc, 0xf8, 0xf7, 0xb8, 0xfb, 0xfd,
  0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc,
  0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc,
  0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc,
  0xfc, 0xde, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc,
  0xfc, 0xfe, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xb3, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc, 0xfc
    };

#undef stdin
#undef stdout
#define stdin 0
#define stdout 1

int main (int argc, unsigned char *argv [])
{
  int ibmtranslate = 0;					    /*  translate IBM code (-i option) */
  int expand_tabs = 0;					    /* -t option: expand tabs to blanks */
  while (1)
    {
    int incount,					    /* length of input line */
        inptr,						    /* index in input line */
        outptr;						    /* and in output line */

    if ((incount = read (stdin, inbuf, LINELENGTH)) < 0)    /* error on input? */
      {
      openlog (argv [0], LOG_ODELAY | LOG_CONS, LOG_AUTH);  /* open system log */
      syslog (LOG_ERR, "Error on stdin: %s, exiting", strerror (errno)); /* and complain */
      return 1;						    /* then die */
      }
    else if (incount == 0)				    /* finished */
      return 0;						    /* no more, no more */
    inptr = outptr = 0;
    while (inptr < incount)				    /* for each character */
      {
      if ((inbuf [inptr] == '\t') && expand_tabs)	    /* got a tab to expand */
	{
	do
	  outbuf [outptr++] = ' ';
	while (outptr & 7);				    /* tab to 8 chara boundaries */
	}
      else	
	{
	if (inbuf [inptr] == '\n')			    /* LF, */
	  outbuf [outptr++] = '\r';			    /* precede with CR */
	if (ibmtranslate)
	  outbuf [outptr++] = translate [(int) inbuf [inptr]];
	else
	  outbuf [outptr++] = inbuf [inptr];
	}
      inptr++;
      }
    if ((write (stdout, outbuf, outptr)) < outptr)	    /* couldn't write? */
      {
      openlog (argv [0], LOG_ODELAY | LOG_CONS, LOG_AUTH);  /* open system log */
      syslog (LOG_ERR,
	      "Error on writing to printer: %s, exiting",
	      strerror (errno));			    /* and complain */
      return 1;						    /* then die */
      }
    }
  }
