.\" This file is in -*- nroff-fill -*- mode
.\" $Id: disks.mm,v 1.2 1996/05/02 17:54:37 grog Exp grog $
.\" $Log: disks.mm,v $
.\" Revision 1.2  1996/05/02  17:54:37  grog
.\" More text
.\"
.\" Revision 1.1  1996/05/02  13:05:37  grog
.\" Initial revision
.\"
.\" Revision 1.20  1996/04/30  11:06:34  grog
.\" Minor mods, checkin for complete "CD Box"
.\"
.\" Revision 1.19  1996/04/11  11:21:50  grog
.\" Checkpoint 11 April
.\"
.\"
.Chapter \*[nchdisks] "Disks and file systems"
One of the most important parts of running any computer system is handling data
on disk.  UNIX file handling varies significantly from other operating systems.
In this chapter, we'll look at some of the more important aspects of disk and
file system handling.  In the next section, we'll look at how FreeBSD structures
its directories, and starting on page
.Sref \*[devices] ,
we'll look at FreeBSD devices.  On page
.Sref \*[second-disk] \&
we'll look into how to add another disk to your system, and on page
.Sref \*[bloody-floppies] \&
we'll look at how to use floppy disks.
.H2 "Directory structure"
.Pn directory-structure
In the course of its evolution, UNIX has changed its directory structure several
times.  The most recent, and probably most far-reaching changes, occurred with
System V.4 and 4.4BSD, both of which made almost identical changes.
.H3 "Creating a partition table"
The next step is to create an MS-DOS style partition table on the disk.  We
looked at the concepts on page
.Sref \*[building-partition-table] ,
but this time we use different tools.  As in MS-DOS, the partitioning program is
called \fIfdisk\fP\|, and the man page is on page
.X "fdisk"
.Sref \*[manpage-fdisk] .
In the following discussion, you'll find a pocket calculator useful.
.P
Depending on what was on the disk before, \fIfdisk\fP\| could get sufficiently
confused to not work correctly.  If you don't format the disk, it's a good idea
to overwrite the beginning of the disk with \fIdd\fP\|:
.Dx
# \f(CBdd if=/dev/zero of=/dev/rsd1 count=100\fP
100+0 records in
100+0 records out
51200 bytes transferred in 1 secs (51200 bytes/sec)
.De
Next, we look at what's on the disk.  This doesn't seem to make much sense,
since we have just overwritten the contents, but \fIfdisk\fP\| will tell us one
thing of importance: the disk geometry.  We'll need this later on.
.Dx
# \f(CBfdisk /dev/rsd1\fP
******* Working on device /dev/rsd1 *******
parameters extracted from in-core disklabel are:
cylinders=967 heads=9 sectors/track=35 (315 blks/cyl)

parameters to be used for BIOS calculations are:
cylinders=967 heads=9 sectors/track=35 (315 blks/cyl)

fdisk: Invalid fdisk partition table found
Warning: BIOS sector numbering starts with sector 1
Information from DOS bootblock is:
The data for partition 0 is:
<UNUSED>
The data for partition 1 is:
<UNUSED>
The data for partition 2 is:
<UNUSED>
The data for partition 3 is:
sysid 165,(FreeBSD/NetBSD/386BSD)
    start 1, size 304604 (148 Meg), flag 80
        beg: cyl 0/ sector 2/ head 0;
        end: cyl 967/ sector 1/ head 0
.De
You'll notice that \fIfdisk\fP\| has decided to put a FreeBSD partition in
partition 3.  That's not what we want in this case: we want an MS-DOS partition
as well This is a small disk by modern standards, only 150 MB, so we'll assign
100 MB to FreeBSD, and leave 50 MB for MS-DOS.  Our resulting partition table
should look like:
.PS
h = .2i
dh = .02i
dw = 1.7i
move right .5i
down
[
	boxht = h; boxwid = 1.7i
	box ht .15i "Master Boot Record"
        box ht .15i "Partition Table"
    P1: box ht .4i
    P2: box ht .4i
    P3:	box ht .4i
    P4:	box ht .4i

	move right 1.8i from P1.ne
	boxwid = 2.2
	down
     C: box ht .25i "MS-DOS \f(CWC:\fP drive"

	move .25i
    FA: box ht .2i "FreeBSD disk label and bootstrap"
        box ht .2i "\fI/dev/sd1a\fP"
        box ht .2i "\fI/dev/sd1s3b\fR: FreeBSD swap"
        box ht .2i "\fI/dev/sd1s3d\fR"
	box ht .2i "\fI/dev/sd1s3e\fR"
        box ht .2i "\fI/dev/sd1s3f\fR"
        box ht .2i "\fI/dev/sd1s3g\fR"
        box ht .2i "\fI/dev/sd1s3h\fR\|: \fI/home\fP\| file system"

	arrow from P1.e to C.w
	arrow from P2.e to FA.w	
	move up .08i from P1.c	"Slice 1 - MS-DOS primary" 
	move down .08i from P1.c  "\fI/dev/sd1s1\fP, 48 MB"
	move up .08i from P2.c	"Slice 2 - FreeBSD" 
	move down .08i from P2.c  "\fI/dev/sd1s2\fP, 100 MB"
	"Slice 3 (unused)" at P3.c
	"Slice 4 (unused)" at P4.c
	]
.PE
.Figure-heading "Partition table on second FreeBSD disk"
.Tn second-disk-partitions
The Master Boot Record and the Partition Table take up the first sector of the
disk, and the rest can be divided between the partitions.  It's easy to make a
mistake in specifying the parameters, and \fIfdisk\fP\|
.X "fdisk"
performs as good as no checking.  You can easily create a partition table that
has absolutely no relationship with reality.  Clearly, it's a good idea to
calculate them in advance.  For each partition, we need to know three things:
.Ls B
.LI
The \fIpartition type\fP, which \fIfdisk\fP\| calls \fIsysid\fP.  This is a
.X "sysid"
number describing what the partition is used for.  FreeBSD partitions have
partition type 165, and modern (release 4 and later) MS-DOS partitions have type
6.
.LI
.X "start sector"
The \fIstart sector\fP, the first sector in the partition.
.LI
.X "end sector"
The \fIend sector\fP\| for the partition.
.Le
In addition, we need to decide which partition is the \fIactive\fP\| partition,
.X "active"
the partition from which we want to boot.  In this case, it doesn't make any
difference, since we won't be booting from the disk, but it's always a good idea
to set it anyway.
.P
We specify the partitions we don't want by giving them a type, start sector and
end sector of 0.  As we have seen, our disk has 304605 sectors, numbered 0 to
304604.  Partitions should start and end on a cylinder boundary.  We want the
FreeBSD partitions to be about 100 MB, or 204,800 sectors, which leaves 99,806
sectors for the MS-DOS partition.  We know the geometry from the \fIfdisk\fP\|
output above: \f(CWcylinders=967 heads=9 sectors/track=35 (315 blks/cyl)\fR.  So
our 204,800 sectors would take up 204800 / 315, or 650.2587302 cylinders.  Since
we want to use this disk primarily for FreeBSD, we'll round this value up to 651
cylinders, or 205,065 sectors.  This leaves 316 cylinders, less the first sector
on the disk, for MS-DOS, a total of (316 * 315) - 1 or 99,539 sectors.  Our
resulting information is:
.TB "sample \fIfdisk\fP\| parameters"
.X "fdisk"
.TS H
center, tab(#) ;
| rfCWp9| rfCWp9| rfCWp9| rfCWp9 | .
_
\fRPartition#\fRPartition#\fRStart#\fRSize
\fRnumber#\fRtype#\fRsector
=
.TH N
1#6#1#99539
2#165#304604#205065
3#0#0
4#0#0
_
.TE
.TB "sample \fIfdisk\fP\| parameters"
.X "fdisk"
.TS
linesize(15), center, tab(#) ;
| rfCWp9| rfCWp9| rfCWp9| rfCWp9 | .
_
\fRPartition#\fRPartition#\fRStart#\fRSize
\fRnumber#\fRtype#\fRsector
=
1#6#1#99539
2#165#304604#205065
3#0#0
4#0#0
_
.TE
.TB "sample \fIfdisk\fP\| parameters"
.X "fdisk"
.TS
linesize(25), center, tab(#) ;
| rfCWp9| rfCWp9| rfCWp9| rfCWp9 | .
_
\fRPartition#\fRPartition#\fRStart#\fRSize
\fRnumber#\fRtype#\fRsector
=
1#6#1#99539
2#165#304604#205065
3#0#0
4#0#0
_
.TE
.P
Now we run \fIfdisk\fP\| in earnest by specifying the \f(CW-i\fP option.  During
.X "fdisk"
this time, you will see messages on the console:
.Dx
sd1: invalid primary partition table: no magic
.De
Since we don't have a partition table yet, they're not surprising.  They're also
completely harmless.
.P
\fIfdisk\fP\| prompts interactively:
.X "fdisk"
.Dx
# \f(CBfdisk -i /dev/rsd1\fP
******* Working on device /dev/rsd1 *******
parameters extracted from in-core disklabel are:
cylinders=967 heads=9 sectors/track=35 (315 blks/cyl)

parameters to be used for BIOS calculations are:
cylinders=967 heads=9 sectors/track=35 (315 blks/cyl)

fdisk: Invalid fdisk partition table found
Warning: BIOS sector numbering starts with sector 1
Information from DOS bootblock is:
The data for partition 0 is:
<UNUSED>
The data for partition 1 is:
<UNUSED>
The data for partition 2 is:
<UNUSED>
The data for partition 3 is:
sysid 165,(FreeBSD/NetBSD/386BSD)
    start 1, size 304604 (148 Meg), flag 80
        beg: cyl 0/ sector 2/ head 0;
        end: cyl 967/ sector 1/ head 0
=== root@daemon (/dev/ttyp1) /usr/home/grog 3 -> fdisk -i sd1
******* Working on device /dev/rsd1 *******
parameters extracted from in-core disklabel are:
cylinders=967 heads=9 sectors/track=35 (315 blks/cyl)

parameters to be used for BIOS calculations are:
cylinders=967 heads=9 sectors/track=35 (315 blks/cyl)

Do you want to change our idea of what BIOS thinks ? [n] \fBENTER \fIpressed\f(CW
fdisk: Invalid fdisk partition table found
Warning: BIOS sector numbering starts with sector 1
Information from DOS bootblock is:
The data for partition 0 is:
<UNUSED>
Do you want to change it? [n] \f(CBy\f(CW
Supply a decimal value for "sysid" [0] \f(CB6\f(CW
Supply a decimal value for "start" [0] \f(CB1\f(CW
Supply a decimal value for "size" [0] \f(CB99539\f(CW
Explicitly specifiy beg/end address ? [n] \fBENTER \fIpressed\f(CW
sysid 6,(Primary 'big' DOS (> 32MB))
    start 1, size 99539 (48 Meg), flag 0
        beg: cyl 0/ sector 2/ head 0;
        end: cyl 315/ sector 35/ head 8
Are we happy with this entry? [n] \f(CBy\f(CW
The data for partition 1 is:
<UNUSED>
Do you want to change it? [n] \f(CBy\f(CW
Supply a decimal value for "sysid" [0] \f(CB165\f(CW
Supply a decimal value for "start" [0] \f(CB99540\f(CW
Supply a decimal value for "size" [0] \f(CB205065\f(CW
Explicitly specifiy beg/end address ? [n] \fBENTER \fIpressed\f(CW
sysid 165,(FreeBSD/NetBSD/386BSD)
    start 99540, size 205065 (100 Meg), flag 0
        beg: cyl 316/ sector 1/ head 0;
        end: cyl 966/ sector 35/ head 8
Are we happy with this entry? [n] \f(CBy\f(CW
The data for partition 2 is:
<UNUSED>
Do you want to change it? [n] \fBENTER \fIpressed\f(CW
The data for partition 3 is:
sysid 165,(FreeBSD/NetBSD/386BSD)
    start 1, size 304604 (148 Meg), flag 80
        beg: cyl 0/ sector 2/ head 0;
        end: cyl 967/ sector 1/ head 0
Do you want to change it? [n] \f(CBy\f(CW

The static data for the DOS partition 3 has been reinitialized to:
sysid 165,(FreeBSD/NetBSD/386BSD)
    start 1, size 304604 (148 Meg), flag 80
        beg: cyl 0/ sector 2/ head 0;
        end: cyl 967/ sector 1/ head 0
Supply a decimal value for "sysid" [165] \f(CB0\f(CW
Supply a decimal value for "start" [1] \f(CB0\f(CW
Supply a decimal value for "size" [304604] \f(CB0\f(CW
Explicitly specifiy beg/end address ? [n] \fBENTER \fIpressed\f(CW
sysid 0,(unused)
    start 0, size 0 (0 Meg), flag 80
        beg: cyl 0/ sector 0/ head 0;
        end: cyl 0/ sector 0/ head 0
Are we happy with this entry? [n] \f(CBy\f(CW
Do you want to change the active partition? [n] \f(CBy\f(CW
Supply a decimal value for "active partition" [3] \f(CB1\f(CW
Are you happy with this choice [n] \f(CBy\f(CW

We haven't changed the partition table yet.  This is your last chance.
parameters extracted from in-core disklabel are:
cylinders=967 heads=9 sectors/track=35 (315 blks/cyl)

parameters to be used for BIOS calculations are:
cylinders=967 heads=9 sectors/track=35 (315 blks/cyl)

Information from DOS bootblock is:
0: sysid 6,(Primary 'big' DOS (> 32MB))
    start 1, size 99539 (48 Meg), flag 0
        beg: cyl 0/ sector 2/ head 0;
        end: cyl 315/ sector 35/ head 8
1: sysid 165,(FreeBSD/NetBSD/386BSD)
    start 99540, size 205065 (100 Meg), flag 80
        beg: cyl 316/ sector 1/ head 0;
        end: cyl 966/ sector 35/ head 8
2: <UNUSED>
3: <UNUSED>
Should we write new partition table? [n] \f(CBy\f(CW
.De
By default, \fIfdisk\fP\| creates a FreeBSD partition in slice 3.  Even though
we didn't have any space left, it created a partition there.  you don't need to
delete it if you don't want to, but since it overlaps the other partitions, it
would probably confuse MS-DOS utilities, so it's a good idea.
.H3 "Labelling the disk"
Now we have a valid MS-DOS partition table.  Now we need to pay more attention
to our FreeBSD slice (slice 1, or MS-DOS partition 2).  It'll make life easier
here to remember a few things:
.Ls B
.LI
From now on, we're just looking at the slice, which we can think of as a logical
disk.  Names like \fIdisk label\fP\| really refer to the slice, but since many
standard terms use the word \fIdisk\fP, we'll continue to use them.
.LI
All offsets are relative to the beginning of the slice, not the beginning of the
disk.  Sizes also refer to the slice and not the disk.
.LI
Depending on the operation, we may need to take the raw disk
(\fI/dev/rsd1c\fP\|) or the block device (\fI/dev/sd1c\fP\|).  Sometimes the
programs can recover if you specify the wrong device, but sometimes they produce
some really strange error messages instead.
.Le
The first thing we need is the disk label, which supplies information about the
disk geometry and the layout of the file systems.  These are two different
things, of course: the overall disk geometry is determined by the kind of disk,
but you have decided (with \fIfdisk\fP\|) what part of the disk represents the
FreeBSD slice, and you also need to define what file systems you want.  As we
saw in \*[chinstall], page
.Sref \*[disklabel-editor] ,
a typical first disk will contain a root file system, swap space and a
\fI/usr\fP\| file system.  On other disks, this arrangement may be different.
.X "/usr"
The arguments in favour of a small root file system are less powerful on other
disks.  On the other hand, you may want to add swap on other disks: firstly,
it's possible that you are a bit low on swap, and secondly, spreading swap over
other disks can improve performance if you are low on memory.  In our example,
let's add 20 MB of swap and make the rest of the slice a \fI/home\fP\| file
.X "/home"
system.  In addition, we need to define the \f(CWc\fP partition, which
represents the whole disk.  Traditionally, data disks use the \f(CWh\fP
partition and not the \f(CWa\fP partition, so we'll stick to that tradition,
though there's nothing to stop you from using the \f(CWa\fP partition if you
prefer.  For each partition, we need to know the start offset, the size and the
file system type.  In summary, the FreeBSD slice we want to create looks like:
.PS
h = .2i
dh = .02i
dw = 1.7i
boxwid = 2.5i
down
[    S: box invis
    FA: box ht .2i "FreeBSD disk label and bootstrap" at S+(1,0)
        box ht .2i "\fI/dev/sd1a\fP (empty)"
        box ht .2i "\fI/dev/sd1s1b\fR: FreeBSD swap, 20 MB"
        box ht .2i "\fI/dev/sd1s1d\fR (empty)"
	box ht .2i "\fI/dev/sd1s1e\fR (empty)"
        box ht .2i "\fI/dev/sd1s1f\fR (empty)"
        box ht .2i "\fI/dev/sd1s1g\fR (empty)"
        box ht .2i "\fI/dev/sd1s1h\fR\|: \fI/home\fP\| file system, 80 MB"
	]
.PE
.Figure-heading "FreeBSD slice on second disk"
.Tn second-disk-freebsd-slice
